﻿Shifter = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

local DEFAULT_OPTIONS = {
	Shifter_SPAMDELAY = 5,
	Shifter_AUTOMOUNT = TRUE,
}

local AceOO = AceLibrary("AceOO-2.0")

function Shifter:OnInitialize()
end

function Shifter:OnEnable()
	self:RegisterDB("ShifterDB")
	self:RegisterDefaults('profile', DEFAULT_OPTIONS)
	self:RegisterChatCommand({"/Shifter", "/Shift"}, ShifterLocals.ChatOptions)
	self.Forms = {}
	self.FormCount = 0
	self:LoadForms()
    	self.lastShift = GetTime()
	self.setTimer = FALSE
	self:RegisterEvent("TAXIMAP_OPENED", "Shift")
	self:RegisterEvent("UI_ERROR_MESSAGE", "AutoShift")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "LoadForms")
end

function Shifter:OnDisable()
end

function Shifter:LoadForms()
	local i
	local n = GetNumShapeshiftForms()
	
	if (self.Forms[ShifterLocals.BearForm] or n ~= self.FormCount) then
		local bear = false
		local moonkin = false
		for i=1,n do
			local _, name = GetShapeshiftFormInfo(i)

			if (name == ShifterLocals.BearForm) then
				bear = true
			end
			if (name == ShifterLocals.MoonkinForm) then
				moonkin = true
			end
			self.Forms[name] = i
		end

		self.FormCount = n
		
		if (not moonkin) then
			self.Forms[ShifterLocals.MoonkinForm] = nil
		end
			
		-- this is an initial load
		if (not arg1) then
			if (not bear) then
				self.Forms[ShifterLocals.BearForm] = nil
			end
			if self.Forms[ShifterLocals.DireBearForm] then
				ShifterLocals.BearForm = ShifterLocals.DireBearForm
				self:Print("Shifter forms loaded: %s (Dire Bear detected)", n)
			else
				self:Print("Shifter forms loaded: %s", n)
			end
		end
	end
end

function Shifter:FindMountBuff()
	local i = 0
	local currBuffTex

	while (1) do
		local bi, _ = GetPlayerBuff(i, "HELPFUL") 
		if bi < 0 then break end
		currBuffTex = GetPlayerBuffTexture(bi)
		for j = 1, table.getn(ShifterLocals.MountTexturs) do
			if (string.find(currBuffTex,ShifterLocals.MountTexturs[j])) then
				return bi
			end
		end
		i = i+1
	end
	return FALSE
end

function Shifter:SetMountItem()
	for b = 0, 4, 1 do
		for s = 1, GetContainerNumSlots(b), 1 do
			local txture = GetContainerItemInfo(b, s)
			if txture then
				for i = 1, table.getn(ShifterLocals.MountTexturs) do
					if (string.find(txture,ShifterLocals.MountTexturs[i])) then
						self.Forms[ShifterLocals.Mount] = { bag = b, slot=s}
						return
					end
				end
			end
		end
	end
end


function Shifter:CurrentForm()
	local cf = ShifterLocals.HumanoidForm
	for i=1,GetNumShapeshiftForms() do
		local _, name, isActive = GetShapeshiftFormInfo(i)
		if isActive == 1 then cf = name end
	end
	if cf == ShifterLocals.HumanoidForm and self.db.profile.Shifter_AUTOMOUNT then
		if self:FindMountBuff() then
			cf = ShifterLocals.Mount
		end
	end
	return cf
end

function Shifter:Shift(f)
	self:SetMountItem()
	if (not f) then
		f = ShifterLocals.HumanoidForm
	end
	local cf = self:CurrentForm()
	-- if we want human form and already human then end
	if (f == ShifterLocals.HumanoidForm and f == cf) then
		return
	end

	if (f == BINDING_NAME_Shifter_BearForm) then
		f = ShifterLocals.BearForm
	end

	if (not self.Forms[f]) then
		if f == ShifterLocals.TravelForm and not (self.Forms[ShifterLocals.TravelForm] or self.Forms[ShifterLocals.AquaticForm] or self.Forms[ShifterLocals.Mount]) then
			self:Print("You don't have any %ss, looks like you are going to be running.", f)
			return
		end
	end
	
	-- argument massaging
	if f == ShifterLocals.TravelForm then
		if self.Forms[ShifterLocals.Mount] and self.db.profile.Shifter_AUTOMOUNT and not UnitAffectingCombat("player") then
			f = ShifterLocals.Mount
		elseif self.Forms[ShifterLocals.AquaticForm] then
			f = ShifterLocals.AquaticForm
		end	
	end

	if cf == f or (f == ShifterLocals.Mount and (cf == ShifterLocals.TravelForm or cf == ShifterLocals.AquaticForm)) then
		local timePassed = GetTime() - self.lastShift
		if timePassed < self.db.profile.Shifter_SPAMDELAY then
			self:Print("Spam Delay %.1f sec left", self.db.profile.Shifter_SPAMDELAY - timePassed)
			return false
		end	
	end

	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", "ShiftSuccess")

	if f ~= ShifterLocals.HumanoidForm and not self.Forms[f] then
		--I asked for a form I don't have
		self:Print("You don't have %s", f)
		return false
	end

	-- want a form and in human form now
	if cf == ShifterLocals.HumanoidForm and self.Forms[f] then
		if f == ShifterLocals.Mount then
			self:RegisterEvent("UI_ERROR_MESSAGE", "ShiftError")
			UseContainerItem(self.Forms[f].bag, self.Forms[f].slot)
		else
			if f == ShifterLocals.AquaticForm then
				self:RegisterEvent("UI_ERROR_MESSAGE", "ShiftError")
			end
			CastShapeshiftForm(self.Forms[f])
		end		
		self.setTimer = TRUE
	else
		-- I'm in an animal form so switch back to humanoid form
		self.setTimer = FALSE
		if cf == ShifterLocals.Mount then
			CancelPlayerBuff(self:FindMountBuff())
		else
			CastShapeshiftForm(self.Forms[cf])
		end
	end
end

function Shifter:ShiftSuccess()
	local form = strfind(arg1, "You gain ([^.]+) Form.")
	if (form) then
		self:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS")
		self:RegisterEvent("UI_ERROR_MESSAGE", "AutoShift")
		if self.setTimer then
			self.lastShift = GetTime()
		else
			self.lastShift = self.lastShift - self.db.profile.Shifter_SPAMDELAY
		end
		self.setTimer = FALSE
	end
end

function Shifter:AutoShift()
	local cf = self:CurrentForm()

	if cf ~= ShifterLocals.HumanoidForm then
		-- tried to talk in animal form
		if arg1 == ERR_CANT_INTERACT_SHAPESHIFTED or
			arg1 == ERR_MOUNT_SHAPESHIFTED or
			arg1 == ERR_NOT_WHILE_SHAPESHIFTED or
			arg1 == ERR_NO_ITEMS_WHILE_SHAPESHIFTED or
			arg1 == SPELL_NOT_SHAPESHIFTED or
			arg1 == SPELL_NOT_SHAPESHIFTED_NOSPACE or
			arg1 == SPELL_FAILED_NOT_SHAPESHIFT or
			arg1 == ERR_EMBLEMERROR_NOTABARDGEOSET or
			((arg1 == ERR_OUT_OF_RAGE or
				arg1 == 'Must be in Bear Form, Dire Bear Form') and
			self:CurrentForm() ~= ShifterLocals.BearForm) or
			((arg1 == ERR_OUT_OF_ENERGY or
				arg1 == 'Must be in Cat Form') and
			self:CurrentForm() ~= ShifterLocals.CatForm) then
			
			self:Shift(ShifterLocals.HumanoidForm)
		end
		--tried to use a bear skill
	elseif cf ~= ShifterLocals.BearForm then
		if arg1 == ERR_OUT_OF_RAGE or
			arg1 == 'Must be in Bear Form, Dire Bear Form' then

				self:Shift(ShifterLocals.BearForm)
		end
		--tried to use a cat skill
	elseif cf ~= ShifterLocals.CatForm then
		if arg1 == ERR_OUT_OF_ENERGY or
					arg1 == 'Must be in Cat Form' then
					
				self:Shift(ShifterLocals.CatForm)
		end
	end
end

function Shifter:ShiftError()
	self:RegisterEvent("UI_ERROR_MESSAGE", "AutoShift")
	-- tried to mount while moving
	if arg1 == SPELL_FAILED_MOVING or 
		arg1 == ERR_NOT_IN_COMBAT or
		arg1 == ERR_CLIENT_LOCKED_OUT then
		if self.Forms[ShifterLocals.TravelForm] then
			self.setTimer = TRUE
			CastShapeshiftForm(self.Forms[ShifterLocals.TravelForm])
			return
		end
	-- tried to mount while swimming
	elseif arg1 == SPELL_FAILED_ONLY_ABOVEWATER then
		if self.Forms[ShifterLocals.AquaticForm] then
			self.setTimer = TRUE
			CastShapeshiftForm(self.Forms[ShifterLocals.AquaticForm])
			return
		end
	-- tried goto aquatic form while on dry land
	elseif arg1 == SPELL_FAILED_ONLY_UNDERWATER then
		if self.Forms[ShifterLocals.TravelForm] then
			self.setTimer = TRUE
			CastShapeshiftForm(self.Forms[ShifterLocals.TravelForm])
			return
		end
	end
	self:UnregisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS")
end

--[[--------------------------------------------------------------------------------
  Command Handlers
-----------------------------------------------------------------------------------]]
function Shifter:SetSpamDelay(i)
	i = tonumber(i)
	self.db.profile.Shifter_SPAMDELAY = i
	self:Print("Spam cutoff set to %s sec", i)
end

function Shifter:GetSpamDelay()
	return self.db.profile.Shifter_SPAMDELAY
end

function Shifter:SetAutoMount()
	if self.db.profile.Shifter_AUTOMOUNT then
		self.db.profile.Shifter_AUTOMOUNT = FALSE
		self:Print("Auto-mount off.")
	else
		self.db.profile.Shifter_AUTOMOUNT = TRUE
		self:Print("Auto-mount on.")
	end
end

function Shifter:GetAutoMount()
	return self.db.profile.Shifter_AUTOMOUNT
end
